import {getAssets} from "./assets.ts";


export class Sprite {
  readonly imageSet: string;
  // spritesheet params
  // image size (px, py)
  readonly px: number; readonly py: number;
  // origin (ox, oy)
  readonly ox: number; readonly oy: number;
  // dimension in cells (cx, cy)
  readonly cx: number; readonly cy: number;
  // number of frames
  readonly nFrames: number;

  constructor(imageSet: string, px: number, py: number, ox: number, oy: number, cx: number, cy: number, nFrames: number) {
    this.imageSet = imageSet;
    this.px = px; this.py = py;
    this.ox = ox; this.oy = oy;
    this.cx = cx; this.cy = cy;
    this.nFrames = nFrames;

    if (this.nFrames < this.cx * this.cy) {
      throw `can't have ${this.nFrames} with a spritesheet dimension of ${this.cx * this.cy}`;
    }
  }

  draw(ctx: CanvasRenderingContext2D, {x, y, ix, xScale, yScale, angle}: {x: number, y: number, ix?: number, xScale?: number, yScale?: number, angle?: number}) {
    ix = ix == undefined ? 0 : ix;
    xScale = xScale == undefined ? 1.0 : xScale;
    yScale = yScale == undefined ? 1.0 : yScale;
    angle = angle == undefined ? 0.0 : angle;

    // ctx.translate(Math.floor(x), Math.floor(y));
    ctx.translate(x, y);
    ctx.rotate(angle * Math.PI / 180);
    ctx.scale(xScale, yScale);
    ctx.translate(-this.ox, -this.oy);

    let me = getAssets().getImage(this.imageSet);
    let srcCx = ix % this.cx;
    let srcCy = Math.floor(ix / this.cx);
    let srcPx = srcCx * this.px;
    let srcPy = srcCy * this.py;
    console.log(`src px and py ${srcPx} ${srcPy}`)
    ctx.drawImage(me, srcPx, srcPy, this.px, this.py, 0, 0, this.px, this.py);
  }
}
